package chapitre10;

import java.net.URL;

import org.apache.catalina.Connector;
import org.apache.catalina.Context;
import org.apache.catalina.Deployer;
import org.apache.catalina.Engine;
import org.apache.catalina.Host;
import org.apache.catalina.logger.SystemOutLogger;
import org.apache.catalina.startup.Embedded;
import org.apache.catalina.Container;

public class EmbeddedTomcat {

  private String path = null;

  private Embedded embedded = null;
  private Host host = null;

  /**
    * Constructeur par dfaut
    *
    */
  public EmbeddedTomcat() {

  }

  /**
    * Acesseur de base dterminant le chemin d'accs au contexte
    *
    * @param        path - e chemin d'accs
    */
  public void setPath(String path) {

    this.path = path;
  }

  /**
    * Acesseur de base retournant le chemin d'accs au contexte
    *
    * @return - le chemin d'accs au contexte
    */
  public String getPath() {

    return path;
  }

  /**
    * Dmarrage du serveur Tomcat
    */
  public void startTomcat() throws Exception {

    Engine engine = null;

    // Configuration du rpertoire racine
    System.setProperty("catalina.home", getPath());

    // Cration d'un serveur intgr
    embedded = new Embedded();
    embedded.setDebug(0);
    embedded.setLogger(new SystemOutLogger());

    // Cration d'un Engine
    engine = embedded.createEngine();
    engine.setDefaultHost("localhost");

    // Cration d'un hte virtuel par dfaut
    host = embedded.createHost("localhost", getPath() + "/webapps");
    engine.addChild(host);

    // Cration du contexte ROOT
    Context context = embedded.createContext("", getPath() + "/webapps/ROOT");
    host.addChild(context);

    // Installation de la hirarchie des conteneurs
    embedded.addEngine(engine);

    // Assemblage et installation d'un connecteur HTTP par dfaut
    Connector connector = embedded.createConnector(null, 8080, false);
    embedded.addConnector(connector);

    // Dmarrage du serveur intgr
    embedded.start();
  }

  /**
    * Cette mthode arrte le serveur Tomccat
    */
  public void stopTomcat() throws Exception {

    // Arrt du serveur
    embedded.stop();
  }

  /**
    * Enregistre un fichier WAR dans le serveur
    *
    * @param contextPath - Le chemin d'accs du contexte dans lequel
    *              l'application sera enregisre
    * @param warFile - L'url du fichier WAR
    */
  public void registerWAR(String contextPath, URL warFile) throws Exception {

    if ( contextPath == null ) {

      throw new Exception("Chemin invalide : " + contextPath);
    }
    String displayPath = contextPath;
    if( contextPath.equals("/") ) {

      contextPath = "";
    }

    if ( warFile == null ) {

      throw new Exception("Fichier WAR invalide : " + warFile);
    }

    Deployer deployer = (Deployer)host;
    Context context = deployer.findDeployedApp(contextPath);

    if (context != null) {

      throw new Exception("Le contexte " + contextPath + " existe dj !");
    }
    deployer.install(contextPath, warFile);
  }

  /**
    * Dsenregistrement d'un fichier WAR
    *
    * @param contextPath - Le chemin du contexte  supprimer
    */
  public void unregisterWAR(String contextPath) throws Exception {

    Context context = host.map(contextPath);
    if ( context != null ) {

      embedded.removeContext(context);
    }
    else {

      throw new Exception("Aucun contexte trouv pour le chemin : "
        + contextPath);
    }
  }

  public static void main(String args[]) {

    try {

      EmbeddedTomcat tomcat = new EmbeddedTomcat();
      tomcat.setPath("C:/Program Files/Apache Group/Tomcat 4.1");

      tomcat.startTomcat();

      URL url =
        new URL("file:C:/Program Files/Apache Group/Tomcat 4.1"
        + "/webapps/eyrollesstruts");

      tomcat.registerWAR("/eyrollesstruts", url);

      Thread.sleep(1000000);

      tomcat.stopTomcat();

      System.exit(0);
    }
    catch( Exception e ) {

      e.printStackTrace();
    }
  }
}
